/*
======================================================
====      Java Classes For Genetic Algorithms     ====
==== (c) Marek Obitko, 1998-1999, obitko@email.cz ====
======================================================

This file is a part of Java library for genetic algorithms with 
visualization. You may use this library or its part free of charge
provided you acknowledge the author (Marek Obitko), and the source 
(Learning Environment for Genetic Algorithms, one of the URLs 
below) in your work where you use it (contact the author, if you 
are in doubt). Moreover I would appreciate if you inform me 
(via email) about your work where you used it 
(this is not compulsory).

THIS LIBRARY IS PROVIDED "AS IS", WITHOUT ANY WARRANTY.

This package was designed for learning environment for GAs, see
http://cs.felk.cvut.cz/~xobitko/ga  or

Author is studying Czech Technical University in Prague, 
Faculty of Electrical Engineering, Department of Computer Engineering.
Major part of this library was written at University of Applied Sciency
in Dresden, Germany.
*/

import java.awt.Graphics;

/** Abstract chromosome which represents any solution used in genetic algorithm.
  * <p>
  * This class defines prototypes for basic operators of genetic algorithm, such as
  * crossover and mutation.
  * <p>
  * @author Marek Obitko, <A HREF="mailto:obitko@email.cz">obitko@email.cz</A>
  * @version 1.0
  */
abstract public class Chromosome implements Cloneable
{

  /** Last crossover point. This is useful for vizualization. Other classes can
    * get information about crossover from this variable. */
  public int lastCrossoverPoint;
  
  /** Fitness of chromosome (solution). Goal of genetic algorithm is 
    * to minimize fitness. */
  public int fitness;
  
  /** Count fitness from information in chromosome and stores it in fitness. 
    * @see #fitness
    */
  abstract public void countFitness();  
  
  /** Performs crossover with given other chromosome.
    * @param other specifies second chromosome for crossover
    * @param probability is crossover probability
    * @return array of two new chromosomes made by crossover 
    */
  abstract public Chromosome[] crossover(Chromosome other, double probability);

  /** Basic crossover. Method crossover(Chromosome other, double probability)
    * will be called.
    * @see crossover(Chromosome, double)
    * @see crossover(Chromosome, double, int)
    */
  public final static int BASIC_CROSSOVER=0;

  /** Performs selected type of crossover with given other chromosome.
    * @param other specifies second chromosome for crossover
    * @param probability is crossover probability
    * @crossoverType type of crossover
    * @return array of two new chromosomes made by crossover 
    * @see #BASIC_CROSSOVER
    */
  public Chromosome[] crossover(Chromosome other, double probability, int crossoverType)
  {
    if(crossoverType==BASIC_CROSSOVER) return crossover(other, probability);
    return null;
  }
  
  
  /** Performs mutation with specified probability.
    * @param probability is mutation probability 
    */
  abstract public void mute(double probability);


  /** Basic mutation. Method mute(double) will be called.
    * @see mute(double)
    * @see mute(double, int)
    */
  public final static int BASIC_MUTATION=0;

  /** Performs mutation with specified probability.
    * @param probability is mutation probability 
    * @mutationType type of mutation
    * @see #BASIC_MUTATION
    */
  public void mute(double probability, int mutationType)
  {
    if(mutationType==BASIC_MUTATION) mute(probability);
  }
  
  /** Randomizes chromosome information. Should call countFitness at the end.
    * @see #countFitness
    */
  abstract public void randomize();    
  
  /** Makes a copy of chromosome. Use rather this than clone - this guarantees really
    * useful copy.
    * @return new chromosome which is an exact copy of this one
    */
  abstract public Chromosome copy();
  
  /** Paints the chromosome to graphics g on coordinates x and y. It can be used for
    * visualization of chromosome. It does not to be defined for using (without
    * visualization), so it is not abstract.
    */
  public void paint(Graphics g, int x, int y)
  {
  }

  /** Paints the chromosome shape without any information 
    * to graphics g on coordinates x and y. It can be used for
    * visualization of empty chromosome shape. It does not to be defined for using 
    * (without visualization), so it is not abstract.
    */
  public void paintEmpty(Graphics g, int x, int y)
  {
  }

  /** Prints contents of chromosome. Useful for example for text information of all
    * the population or for debugging.
    */
  public void printSelf()
  {
    System.out.println("Abstract chromosome");
  }

}