/*
======================================================
====      Java Classes For Genetic Algorithms     ====
==== (c) Marek Obitko, 1998-1999, obitko@email.cz ====
======================================================

This file is a part of Java library for genetic algorithms with 
visualization. You may use this library or its part free of charge
provided you acknowledge the author (Marek Obitko), and the source 
(Learning Environment for Genetic Algorithms, one of the URLs 
below) in your work where you use it (contact the author, if you 
are in doubt). Moreover I would appreciate if you inform me 
(via email) about your work where you used it 
(this is not compulsory).

THIS LIBRARY IS PROVIDED "AS IS", WITHOUT ANY WARRANTY.

This package was designed for learning environment for GAs, see
http://cs.felk.cvut.cz/~xobitko/ga  or

Author is studying Czech Technical University in Prague, 
Faculty of Electrical Engineering, Department of Computer Engineering.
Major part of this library was written at University of Applied Sciency
in Dresden, Germany.
*/


import java.awt.Graphics;
import java.awt.Canvas;
import java.awt.Dimension; 
import java.awt.Color;
import java.awt.Image;

/** Component for showing crossover on number chromosome. It is not very interesting
  * component, because it only defines, where everything should be displayed. It is very
  * dependent on type of chromosome. This basic component shows crossover of number
  * chromosome only.
  * <p> 
  * It uses double buffering to prevent blinking.
  * @see Population
  * @see Population#makeStep()
  * <p>
  * @author Marek Obitko, <A HREF="mailto:obitko@email.cz">obitko@email.cz</A>
  * @version 1.0
  */
public class CrossoverCanvas extends Canvas
{

  /** Size of component */
  protected Dimension mySize;
  
  /** Population being displayed on the graph. */
  protected Population population=null;
  
  /** Graphics of component */
  protected Graphics gr;
  
  /** Graphics of offline buffer */
  protected Graphics offgr;
  
  /** Image of offline buffer */
  protected Image offscreen;
  
  /** Is it showing anything? */
  protected boolean animating=false;
  
  /** Is elitism shown now? */
  protected boolean elitism=false;
  
  /** index in the new population */
  protected int index=0;
  
  /** index of operation in creation of the new population */
  protected int crossover=0;
  
  /** Constructor which makes standart size graph */
  public CrossoverCanvas()
  {
    this(690,275);
  }
  
  /** Constructor where you can specify size of component */
  public CrossoverCanvas(int xsize, int ysize)
  {
    super();
    mySize=new Dimension(xsize,ysize);
  }
 
  /** Sets if the animation should be performed. Set false to not animate. */
  public void animate(boolean toAnimate)
  {
    animating=toAnimate;
    paint();
  }
  
  /** Draws labels on graphics.
    * @param g graphics which the labels is drawn on
    */
  protected void drawLabels(Graphics g)
  {
    g.drawString("Present population",87,245);
    g.drawString("New population",510,245);
    g.drawString("Parents",262,245);
    g.drawString("New",340,245);
    g.drawString("Offspring",327,258);
    g.drawString("Mutated",397,245);
    g.drawString("Offspring",392,258);
  }
  
  /** Shows elitism. */
  public void makeElitism(int anIndex)
  {
    index=anIndex;
    crossover=0;
    elitism=true;
    paint();
  }

  /** Show parents. */
  public void showParents()
  {
    crossover=1;
    elitism=false;
    paint();
    if(population.parent1.lastCrossoverPoint==-1)
      { 
        gr.drawString("No crossover",330,130);
        gr.drawString("(childrens will be",322,150);
        gr.drawString("copies of parents)",320,170);
      }  
     else 
      gr.drawString("Crossover point",320,9+population.parent1.lastCrossoverPoint*7);
  }
  
  /** Shows children */
  public void showChildren()
  {
    crossover=2;
    paint();
  }
  
  /** Shows mutated children */
  public void showMutated()
  {
    crossover=3;
    paint();
  }
  
  /** Shows new children without mutation. */
  public void showNewChildren(int anIndex)
  {
    crossover=4;
    elitism=false;
    int end=(index-1)*10+472;
    for(int j=405; j<end; j+=10) //index
      {
        paint();
        population.child[0].paint(gr,j,5);
        population.child[1].paint(gr,j+20,5);
      }
    index=anIndex;
    crossover=0;
    paint();
  }
  
  /** Shows sorted new population. */
  public void showSort()
  {
    crossover=0;
    paint();
    gr.drawString("has been sorted",95,258);
  }
  
  /** Shows removing old population and accepting the new one. */
  public void removeOldPopulation()
  {
    crossover=0;
    elitism=false;
    for(int j=472; j>62; j-=10) // animation loop
      {
        offgr.setColor(getBackground()); // erase all
        offgr.fillRect(0, 0, mySize.width, mySize.height);
        offgr.setColor(Color.black);
        int i;
        for(i=0; i<population.size; i++)
          population.pop[i].paintEmpty(offgr, i*10+62, 5); // old population empty
        for(i=0; i<population.size; i++)
          population.newpop[i].paint(offgr, i*10+j, 5);  // new one being shifted
        offgr.setColor(Color.black);  // box around
        offgr.drawRect(0, 0, mySize.width - 1, mySize.height - 1);
        offgr.drawString("Removing old population and accepting the new one",230,258);
        gr.drawImage(offscreen, 0, 0, this);
      }
    index=0;    
    paint();
  }
  
  /** Shows the position of the chromosome in a graph (with an arrow to graph). */
  public void showPosition(Graphics g, int x, int height, Chromosome C)
  {
    x+=3;
    g.setColor(Color.green);
    g.drawLine(x,233,x,265+height);
    int val=(int)(690*((NumberChromosome)C).value()/4294967296L);
    g.drawLine(x,265+height,val,265+height);
    g.drawLine(val,265+height,val,273);
    g.drawLine(val,273,val-2,270);
    g.drawLine(val,273,val+2,270);
    g.setColor(Color.black);
  }
  
  /** Draws population on graphics.
    * @param g graphics which the solutions are drawn on
    */
  protected void drawPopulation(Graphics g)
  {
    if(population==null) return; 
    int i;
    for(i=0; i<population.size; i++) // old population
      population.pop[i].paint(g, i*10+62, 5);
    for(i=0; i<index; i++)
      population.newpop[i].paint(g, i*10+472, 5); // part of new population
    for(; i<population.size; i++)
      population.pop[i].paintEmpty(g, i*10+472, 5); // empty part of new population
    if(elitism)  // if elitism is now performed
      {
        g.drawString("Elitism",320,110);
        g.drawString("(the best chromosomes are just copied)",240,130);
        showPosition(g, (index-1)*10+472, 0, population.newpop[index-1]);
      }
    switch(crossover) // number of step in crossover and mutation
      {
        case 0: break; // nothing
        case 4: g.drawString("Accepting new offspring",240,140); // accepting new offspring
                break;
        case 3: population.child[0].paint(g,405,5); // after mutation
                population.child[1].paint(g,425,5);
                showPosition(g, 425, 0, population.child[1]); // shows position of mutated
                showPosition(g, 405, -3, population.child[0]);
                for(int c=0; c<2; c++)  // show differences after mutation
                  for(i=0; i<32; i++)
                    if(((NumberChromosome)population.child[c]).bit[i]!=
                       ((NumberChromosome)population.oldchild[c]).bit[i])
                          g.fillRect(c*20+400,7+i*7,4,4); // box next to the offspring
        case 2: population.oldchild[0].paint(g,340,5);  // offspring without mutation
                population.oldchild[1].paint(g,360,5);
                if(crossover==2)  // position
                  {
                    showPosition(g, 340, 0, population.oldchild[0]);
                    showPosition(g, 360, -3, population.oldchild[1]);
                  }
        case 1: population.parent1.paint(g,270,5);  // show parents
                population.parent2.paint(g,290,5);
                if(crossover==1)
                  {
                    showPosition(g, 270, 0, population.parent1); // position of parents
                    showPosition(g, 290, -3, population.parent2);
                    int idx[]=population.parentIndex;
                    g.fillRect(idx[0]*10+63,2,7,2);  // show where the parents are from
                    g.fillRect(idx[1]*10+63,2,7,2);  // in the old population
                    g.fillRect(idx[0]*10+63,232,7,2);
                    g.fillRect(idx[1]*10+63,232,7,2);
                  }
                g.setColor(Color.green);  
                g.fillRect(265,5+population.parent1.lastCrossoverPoint*7,173,2);
                    // show crossover point 
                g.setColor(Color.black);
                break;
      }
  }
  
  /** Initializes double buffering.
    * This should be called before using method paint.
    * @see #paint
    */
  public void initPaint()
  {
    gr = this.getGraphics();
    offscreen = this.createImage(mySize.width, mySize.height);
    offgr = offscreen.getGraphics();
  }
  
  /** This method acts like repaint(), but it prevents blinking by buffering
    * an image.
    */
  public void paint()
  {
    if(offgr==null) initPaint();
    offgr.setColor(getBackground());
    offgr.fillRect(0, 0, mySize.width, mySize.height);
    offgr.setColor(Color.black);
    paint(offgr);
    gr.drawImage(offscreen, 0, 0, this);
    //paint(getGraphics());
  }
  
  /** Paints graph and solutions on it (if there are any). */
  public void paint(Graphics g) 
  {
    g.setColor(Color.black);  // box around
    g.drawRect(0, 0, mySize.width - 1, mySize.height - 1);
    if(!animating) return;
    drawLabels(g);
    drawPopulation(g);  
  }
  
  /** Sets population for drawing.
    * @param aPopulation population, which will be shown on the graph; use null for
    *        drawing no population on graph
    */ 
  public void setPopulation(Population aPopulation)
  {
    population=aPopulation;
  }
  
  public Dimension getPreferredSize()
  {
    return mySize;
  }

  public Dimension getMaximumSize()
  {
    return mySize;
  }

  public Dimension getMinimumSize()
  {
    return mySize;
  }

}