/*
======================================================
====      Java Classes For Genetic Algorithms     ====
==== (c) Marek Obitko, 1998-1999, obitko@email.cz ====
======================================================

This file is a part of Java library for genetic algorithms with 
visualization. You may use this library or its part free of charge
provided you acknowledge the author (Marek Obitko), and the source 
(Learning Environment for Genetic Algorithms, one of the URLs 
below) in your work where you use it (contact the author, if you 
are in doubt). Moreover I would appreciate if you inform me 
(via email) about your work where you used it 
(this is not compulsory).

THIS LIBRARY IS PROVIDED "AS IS", WITHOUT ANY WARRANTY.

This package was designed for learning environment for GAs, see
http://cs.felk.cvut.cz/~xobitko/ga  or

Author is studying Czech Technical University in Prague, 
Faculty of Electrical Engineering, Department of Computer Engineering.
Major part of this library was written at University of Applied Sciency
in Dresden, Germany.
*/

import java.applet.Applet;
import java.awt.*;
import java.awt.event.*;


/** Shows basic genetic algorithm on a simle example and allows to change some basic 
  * parameters of genetic algorithm. It also displays progress of the solution.
  * <p>
  * @see BasicGA
  * <p>
  * @author Marek Obitko, <A HREF="mailto:obitko@email.cz">obitko@email.cz</A>
  * @version 1.0
  */
public class ParamsBasicGA extends Applet 
       implements ActionListener, ItemListener, Runnable
{
  /** Pause between generations in run of algorithm */
  protected int STEP_PAUSE=0;
  
  /** Size of population (number of chromosomes), should be even number */
  final private int POP_SIZE=16;
                                 
  /** Thread for running of genetic algorithm in background */
  Thread algThread=null;
                                   
  /** Population of solutions in shown genetic algorithm */
  Population population;

  /** Component for drawing graph and solutions on it */
  GraphCanvas Graph=new GraphCanvas();
  
  /** Button Start */
  Button Start=new Button(" Start ");

  /** Button Step */
  Button Step=new Button(" Step ");

  /** Button Stop */
  Button Stop=new Button(" Stop ");

  /** Button Continue */
  Button Continue=new Button(" Continue ");

  /** Button Reset */
  Button Reset=new Button(" Reset ");
  
  
  Checkbox Elitism=new Checkbox("Elitism");
  
  ShifterLabel Speed=new ShifterLabel("Speed", 100, 10, 10, 1000);

  ShifterLabel crossover=new ShifterLabel("Crossover");
  
  ShifterLabel mutation=new ShifterLabel("Mutation");
  
  ProgressGraph Progress=new ProgressGraph(690,140,2);
  
  /** indicates if GA has been initialized */
  protected boolean initialized=false;
  
  /** crossover probability */
  protected int crossoverProb=30;
  
  /** mutation probability */
  protected int mutationProb=4;
  
  
  /** Initialization of applet and genetic algorithm */
  public void init() 
  {
    setBGColor();
    NumberChromosome sample=new NumberChromosome(32);
    sample.randomize();
    sample.countFitness();
    population=new Population(sample, POP_SIZE);
    population.randomize();
    add(Graph);
    add(Progress);

    Panel controls=new Panel();
    controls.setLayout(new BorderLayout());
    Panel buttons=new Panel();
    buttons.setLayout(new FlowLayout());
    Panel shifters=new Panel(); 
    shifters.setLayout(new FlowLayout());

    buttons.add(Start); 
    Start.addActionListener(this);
    buttons.add(Step);
    Step.addActionListener(this);
    buttons.add(Stop);  
    Stop.addActionListener(this);
    buttons.add(Continue);  
    Continue.addActionListener(this);
    buttons.add(Reset);
    Reset.addActionListener(this);
    shifters.add(Speed);
    shifters.add(crossover);
    crossover.setValue(crossoverProb);
    shifters.add(mutation);
    mutation.setValue(mutationProb);
    shifters.add(Elitism);
    Elitism.addItemListener(this); 

    controls.add("North",buttons);
    controls.add("South",shifters);
    add(controls);
      
    Progress.deleteAll();
    Graph.initPaint(); 
    population.elitism=Elitism.getState(); 
  }

  /** Sets the background color, which is specified in Applete parameter BGCOLOR.
    * For using, just add to applet tag in HTML something like
    * "<"PARAM name=BGCOLOR value="00ff40">".
    */
  protected void setBGColor()
  {
    try 
      {
        Color bg = new Color(Integer.parseInt(getParameter("BGCOLOR"), 16));
        setBackground(bg);
      } catch (NumberFormatException e) {}
        catch (NullPointerException e) {}
  }
    
  /** Handles actions from buttons */
  public void actionPerformed(ActionEvent e)
  {
    String command=e.getActionCommand();
    if(command.equals(Start.getLabel())) actionStart();
    if(command.equals(Stop.getLabel())) actionStop();
    if(command.equals(Reset.getLabel())) actionReset();
    if(command.equals(Step.getLabel())) actionStep();
    if(command.equals(Continue.getLabel())) actionContinue();
  }

  /** Handles actions from checkboxes and item selectors */
  public void itemStateChanged(ItemEvent e) 
  {
    population.elitism=Elitism.getState(); 
  }

  /** Button Start starts genetic algorithm (as a thread) */
  protected void actionStart()
  {
    initialized=false;
    if(algThread!=null) return;
    algThread = new Thread(this);
    algThread.start();
  }

  /** Button Continue continues running genetic algorithm (as a thread) */
  protected void actionContinue()
  {
    initialized=true;
    if(algThread!=null) return;
    algThread = new Thread(this);
    algThread.start();
  }
  
  /** Button Reset resets genetic algorithm */
  protected void actionReset()
  {
    Graph.setPopulation(population);
    population.randomize();
    Progress.deleteAll();
    Progress.paint();
    Graph.paint();
    changes();
  }
  
  /** Button Stop stops genetic algorithm (the thread) */
  protected void actionStop()
  {
    if(algThread!=null)
      {
        algThread.stop();
        algThread=null;
      }
    Graph.paint();
  }
  
  /** Button Step performs one step (on generation) of genetic algorithm */
  protected void actionStep()
  {
    changes();
    population.newGeneration(crossoverProb, mutationProb);
    Progress.addValue((int)(population.fitnessSum/population.size),1);
    Progress.addValue(population.pop[0].fitness,0);
    Graph.paint();
    Progress.paint();
  }
  
  protected void changes()
  {
    if(Speed.changed()) STEP_PAUSE=Speed.getValue();
    if(crossover.changed()) crossoverProb=crossover.getValue();
    if(mutation.changed()) mutationProb=mutation.getValue();
  }
  
  /** Defines run of genetic algorithm thread */
  public void run()
  {
    if(!initialized)
      {
        Graph.setPopulation(population);
        Graph.paint();
        Progress.deleteAll();
        Progress.paint();
        changes();
        try { Thread.sleep(STEP_PAUSE);
            } catch (InterruptedException e) {}
      }      
    while(true)
      {
         actionStep();   
         try { Thread.sleep(STEP_PAUSE);
             } catch (InterruptedException e) {}
      }
  }
  
  /** Finishing of the applet and destroying thread. */
  public void destroy()
  {
    if(algThread!=null)
      {
        algThread.stop();
        algThread=null;
      }
    super.destroy();
  }

  /** Finishing of the applet and destroying thread.
    * @exception Throwable successor can throw an exception
    */
  protected void finalize() throws Throwable
  {
    if(algThread!=null)
      {
        algThread.stop();
        algThread=null;
      }
    super.finalize();
  }

  /** Returns information about parameters of applet. */
  public String[][] getParameterInfo() 
  {
    String[][] info = 
     {
      {"BGCOLOR", "color (6 hexa digits)", "Background color, for example \"00ff40\"."},
     };
        return info;
  }

  /** Returns information about applet. */
  public String getAppletInfo() 
  {
    return "Title: Genetic algorithm with parameters \n"+
           "Author: Marek Obitko, obitko@email.cz \n"+
           "Applet shows GA and allows to change parameters of GA. Also shows progress graph.";
  }

  /** For running of applet without browser or appletviewer (it is then possible
    * to run this applet as a class directly).
    * @see GAApplication
    */
  public static void main(String[] args)
  {
    Applet applet = new ParamsBasicGA();
    Frame frame = new GAApplication("Basic genetic algorithm with possibility of parameters change", 
                                     applet, 720, 430); 
  }                                     
}