/*
======================================================
====      Java Classes For Genetic Algorithms     ====
==== (c) Marek Obitko, 1998-1999, obitko@email.cz ====
======================================================

This file is a part of Java library for genetic algorithms with 
visualization. You may use this library or its part free of charge
provided you acknowledge the author (Marek Obitko), and the source 
(Learning Environment for Genetic Algorithms, one of the URLs 
below) in your work where you use it (contact the author, if you 
are in doubt). Moreover I would appreciate if you inform me 
(via email) about your work where you used it 
(this is not compulsory).

THIS LIBRARY IS PROVIDED "AS IS", WITHOUT ANY WARRANTY.

This package was designed for learning environment for GAs, see
http://cs.felk.cvut.cz/~xobitko/ga  or

Author is studying Czech Technical University in Prague, 
Faculty of Electrical Engineering, Department of Computer Engineering.
Major part of this library was written at University of Applied Sciency
in Dresden, Germany.
*/

import java.awt.AWTEvent;
import java.awt.Canvas;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Label;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseListener;

/** Component for mouse input of percentage scale.
  * <p> It can be used for any ranges and supports handling one label directly.
  * <p>
  * @author Marek Obitko, <A HREF="mailto:obitko@email.cz">obitko@email.cz</A>
  * @version 1.0
  */
public class Shifter extends Canvas 
       implements MouseMotionListener, MouseListener
{
  /** value of the shifter */
  protected int value = 0;
  
  /** width of the box in the shifter */
  protected int boxWidth = 10;

  /** range of the shifter (0 till the value of this) */
  protected double mRange = 100;
  
  /** label, where value will be displayed */
  private Label label=null;
  
  /** size of the component */
  protected Dimension mySize;
  
  /** if value has been changed */
  protected boolean change=true;
  
  /** Constructor, where only size of component is specified. */
  public Shifter(int width, int height)
  {
    this(width, height, 10, 100);
  }

  /** Constructor, where only size of component and of the box in the shifter 
    * is specified. */
  public Shifter(int width, int height, int inBoxWidth)
  {
    this(width, height, inBoxWidth, 100);
  }
  
  /** Constructor, where size of component, size of the box in the shifter 
    * and range multiplier is specified. 
    * @param width width of the shifter
    * @param height width of the shifter
    * @param inBoxWidth specifies width of inside box (which is shifted)
    * @param range specifies range of the shifter - it will be 0 till the value of this 
    *        parameter (standart is 100 - like percentage scale)
    */
  public Shifter(int width, int height, int inBoxWidth, double range) 
  {
    super();
    mySize=new Dimension(width, height);
    setSize(width,height);
    boxWidth=inBoxWidth;
    mRange=range;
    addMouseMotionListener(this);
    addMouseListener(this);
  }

  /** Returns value of the shifter.
    * @return integer value in the range from 0 to Range
    */
  public int getValue() 
  {
    return (int)(mRange*value/(mySize.width-boxWidth-1));
  }

  /** Sets new value for the shifter. */
  public void setValue(int newValue)
  {
    value=(int)(newValue*(mySize.width-boxWidth-1)/mRange);
    repaint();
  }
  
  /** Sets label for displaying the value. Use null for no label. */
  public void setValueLabel(Label label) 
  {
    this.label = label;
  }

  /** Paints the area of component shifter */
  public void paint(Graphics g) 
  {
    Dimension d = this.getSize();
    int px, py;

    g.setColor(Color.black);
    g.drawRect(0, 0, mySize.width-1, mySize.height-1);
    g.fillRect(value, 0, boxWidth, mySize.height-1);
    if(label != null) label.setText(String.valueOf(getValue()));
  }

  public Dimension getPreferredSize() 
  {
    return mySize;
  }

  public Dimension getMinimumSize() 
  {
    return mySize;
  }

  public Dimension getMaximumSize() 
  {
    return mySize;
  }

  /** Answers if the value of shifter has been changed from the time of last asking. */
  public boolean changed()
  {
    if(change)
      {
        change=false;
        return true;
      }
    return false;  
  }
    
  protected boolean mouseDown=false;
  
  /** Handles shifting of the box in the shifter (and changing of the value). */
  public void mouseDragged(MouseEvent e) 
  {
    if(!mouseDown) return;

    int y=e.getY();
    if(y<0) return;
    if(y>mySize.height) return;
    int x=e.getX();
    if(x<0) return;
    if(x>mySize.width) return;
    
    int nvalue = x-(int)(boxWidth/2); // get it in the middle of the box
    if(nvalue<0) nvalue=0;
    if(nvalue>mySize.width-boxWidth-1) nvalue=mySize.width-boxWidth-1;
    
    if (value != nvalue)  // changed ?
      {
        value = nvalue;
        change=true;
        repaint();
      }
  }

  public void mouseMoved(MouseEvent e) 
  {
  }

  public void mouseClicked(MouseEvent e)  
  {
  }
 
  public void mouseEntered(MouseEvent e)  
  {
  }
 
  public void mouseExited(MouseEvent e)  
  {
  }
 
  public void mousePressed(MouseEvent e)  
  {
    int y=e.getY();
    if(y<0) return;
    if(y>mySize.height) return;
    int x=e.getX();
    if(x<0) return;
    if(x>mySize.width) return;
    mouseDown=true;
    mouseDragged(e);
    //System.out.println(e);
  }
  
  public void mouseReleased(MouseEvent e)
  {
    mouseDown=false;
  }
}