/*
======================================================
====      Java Classes For Genetic Algorithms     ====
==== (c) Marek Obitko, 1998-1999, obitko@email.cz ====
======================================================

This file is a part of Java library for genetic algorithms with 
visualization. You may use this library or its part free of charge
provided you acknowledge the author (Marek Obitko), and the source 
(Learning Environment for Genetic Algorithms, one of the URLs 
below) in your work where you use it (contact the author, if you 
are in doubt). Moreover I would appreciate if you inform me 
(via email) about your work where you used it 
(this is not compulsory).

THIS LIBRARY IS PROVIDED "AS IS", WITHOUT ANY WARRANTY.

This package was designed for learning environment for GAs, see
http://cs.felk.cvut.cz/~xobitko/ga  or

Author is studying Czech Technical University in Prague, 
Faculty of Electrical Engineering, Department of Computer Engineering.
Major part of this library was written at University of Applied Sciency
in Dresden, Germany.
*/

import java.applet.Applet;
import java.awt.*;
import java.awt.event.*;


/** Shows basic genetic algorithm on solving TSP (Travelling salesman problem).
  * <p>
  * Not developed, TSPGAAll is better - use rather that one.
  * @see TSPGAAll
  * <p>
  * @author Marek Obitko, <A HREF="mailto:obitko@email.cz">obitko@email.cz</A>
  * @version 1.0
  */
public class TSPGA extends Applet implements ActionListener, Runnable
{
  /** Pause between generations in run of algorithm */
  final private int STEP_PAUSE=0;
  
  /** Size of population (number of chromosomes), should be even number */
  final private int POP_SIZE=16;
                                 
  /** Thread for running of genetic algorithm in background */
  Thread algThread=null;
                                   
  /** Population of solutions in shown genetic algorithm */
  Population population;

  TSPInstance points=new TSPInstance(50);
  
  protected boolean gaInitialized;
  
  /** Component for drawing graph and solutions on it */
  TSPPathCanvas Path=new TSPPathCanvas(400,400,points);
    
  /** Button Start */
  Button Start=new Button(" Start ");

  /** Button Step */
  Button Step=new Button(" Step ");

  /** Button Stop */
  Button Stop=new Button(" Stop ");

  /** Button Reset */
  Button Reset=new Button(" Reset ");
  
  TextField popNo = new TextField("0",5);
  
  /** Initialization of applet and genetic algorithm */
  public void init()
  {
    gaInitialized=false;
    population=null;
    add(Path);
    add(Start); 
    Start.addActionListener(this);
    add(Step);
    Step.addActionListener(this);
    add(Stop);  
    Stop.addActionListener(this);
    add(Reset);
    Reset.addActionListener(this);
    add(popNo);
    Path.randomize(5);
    Path.drawPath=true;
  }
    
  /** Handles actions from buttons */
  public void actionPerformed(ActionEvent e)
  {
    String command=e.getActionCommand();
    if(command.equals(Start.getLabel())) actionStart();
    if(command.equals(Stop.getLabel())) actionStop();
    if(command.equals(Reset.getLabel())) actionReset();
    if(command.equals(Step.getLabel())) actionStep();
  }

  /** Button Start starts genetic algorithm (as a thread) */
  protected synchronized void actionStart()
  {
    if(algThread!=null) return;
    algThread = new Thread(this);
    algThread.start();
  }
  
  /** Button Reset resets genetic algorithm */
  protected synchronized void actionReset()
  {
    if(!gaInitialized) actionInit();
    population.randomize();
    Path.setPath(((TSPChromosome)(population.pop[0])).perm);
  }
  
  /** Button Stop stops genetic algorithm (the thread) */
  protected synchronized void actionStop()
  {
    //Path.drawPath=false;
    if(algThread!=null)
      {
        algThread.stop();
        algThread=null;
      }
    Path.paint();
    Path.canAdd=true;
  }
  
  /** Button Step performs one step (on generation) of genetic algorithm */
  protected synchronized void actionStep()
  {
    if(!gaInitialized) actionInit();
    if(Path.changed()) actionInit();
    population.newGeneration();
    Path.setPath(((TSPChromosome)(population.pop[0])).perm);
  }
  
  /** Creates new population. */
  protected synchronized void actionInit()
  {
    //System.out.println("INIT");
    Path.changed(); // change will be handled now, so clear the semaphore
    //Path.canAdd=false;
    Path.tspi.computeDistanceMatrix();
    TSPChromosome sample=new TSPChromosome(Path.tspi.size, points);
    sample.randomize();
    sample.countFitness();
    population=new Population(sample, POP_SIZE);
    population.popNo=popNo;
    population.randomize();
    Path.setPath(((TSPChromosome)(population.pop[0])).perm);
    Path.drawPath=true;
    gaInitialized=true;
    Path.paint();
  }
  
  /** Defines run of genetic algorithm thread */
  public void run()
  {
    //Path.setPopulation(population);
    //Path.paint();
    //if(!gaInitialized) actionInit();
    try { Thread.sleep(STEP_PAUSE);
        } catch (InterruptedException e) {}
    while(true)
      {
         actionStep();   
         try { Thread.sleep(STEP_PAUSE);
             } catch (InterruptedException e) {}
      }
  }

  /** Finishing of the applet and destroying thread. */
  public void destroy()
  {
    if(algThread!=null)
      {
        algThread.stop();
        algThread=null;
      }
    super.destroy();
  }

  /** Finishing of the applet and destroying thread.
    * @exception Throwable successor can throw an exception
    */
  protected void finalize() throws Throwable
  {
    if(algThread!=null)
      {
        algThread.stop();
        algThread=null;
      }
    super.finalize();
  }

  /** Returns information about parameters of applet. */
  public String[][] getParameterInfo() 
  {
    String[][] info = 
     {
      {"BGCOLOR", "color (6 hexa digits)", "Background color, for example \"00ff40\"."},
     };
        return info;
  }

  /** Returns information about applet. */
  public String getAppletInfo() 
  {
    return "Title: Genetic algorithm for TSP \n"+
           "Author: Marek Obitko, obitko@email.cz \n"+
           "Applet shows GA solving travelling salesman problem.";
  }


  /** For running of applet without browser or appletviewer (it is then possible
    * to run this applet as a class directly).
    * @see GAApplication
    */
  public static void main(String[] args)
  {
    Applet applet = new BasicGA();
    Frame frame = new GAApplication("Genetic Algorithm for TSP", applet, 720, 250);
  }
}