/*
======================================================
====      Java Classes For Genetic Algorithms     ====
==== (c) Marek Obitko, 1998-1999, obitko@email.cz ====
======================================================

This file is a part of Java library for genetic algorithms with 
visualization. You may use this library or its part free of charge
provided you acknowledge the author (Marek Obitko), and the source 
(Learning Environment for Genetic Algorithms, one of the URLs 
below) in your work where you use it (contact the author, if you 
are in doubt). Moreover I would appreciate if you inform me 
(via email) about your work where you used it 
(this is not compulsory).

THIS LIBRARY IS PROVIDED "AS IS", WITHOUT ANY WARRANTY.

This package was designed for learning environment for GAs, see
http://cs.felk.cvut.cz/~xobitko/ga  or

Author is studying Czech Technical University in Prague, 
Faculty of Electrical Engineering, Department of Computer Engineering.
Major part of this library was written at University of Applied Sciency
in Dresden, Germany.
*/

/** Instance of Travelling Salesman Problem
  * <p>
  * Mantains information about cities and paths (in complete graph).
  * <p>
  * @author Marek Obitko, <A HREF="mailto:obitko@email.cz">obitko@email.cz</A>
  * @version 1.0
  */
public class TSPInstance
{
  /** maximum for arrays*/
  public int max;
  
  /** real number of cities */
  public int size;

  /** x coordinates */
  public int x[];

  /** y coordinates */
  public int y[];

  /** minimum distance matrix */
  int dist[][];

  /** if distance matrix has been computed */
  public boolean dmComputed;
  
  /** Creates new instance with specified maximum of cities.
    * @param	max	maximum of cities
    */
  public TSPInstance(int max)
  {
    this.max = max;
    size=0;
    x = new int[max];
    y = new int[max];
    dist = new int[max][max];
    dmComputed=false;
  }
  
  /** Changes size of the instance
    * @param newMax new maximum
    */
  public void resize(int newMax)
  {
    int oldx[]=x;
    int oldy[]=y;
    x=new int[newMax];
    y=new int[newMax];
    dist=new int[newMax][newMax];
    dmComputed=false;
    int to=max;
    if(newMax<to) to=newMax;
    max=newMax;
    for(int i=0; i<to; i++) 
      {
        x[i]=oldx[i];
        y[i]=oldy[i];
      }
    size=to;  
  }
  
  /** Deletes all cities.
    */
  public void clearCities()
  {
    size=0;
    dmComputed=false;
  }
  
  /** Adds a new city with specified coordinates.
    *
    * @param	x	x coordinate
    * @param	y	y coordinate
    */
  public void addCity(int x, int y)
  {
    if(size==max) return;
    this.x[size]=x;
    this.y[size]=y;
    size++;
    dmComputed=false;
    //computeDistanceMatrix();
  }
  
  private final static int NEAR_SENSITIVITY=7;
  
  /** Test, if specified coordinates are near some city. If so, index of this city
    * is returned, otherwise -1 is returned.
    */
  public int nearCity(int xx, int yy)
  {
    for(int i=0; i<size; i++)
      if(Math.sqrt((x[i]-xx)*(x[i]-xx)+(y[i]-yy)*(y[i]-yy))<NEAR_SENSITIVITY)
        return i;
    return -1;
  }
  
  /** Deletes city with specified index. */
  public void deleteCity(int index)
  {
    size--;
    for(int i=index; i<size; i++)
      {
        x[i]=x[i+1];
        y[i]=y[i+1];
      }
    dmComputed=false; 
  }
  
  /** Adds or delets city, depending on distance from old cities.
    * @see #nearCity(int, int)
    */
  public void checkCity(int x, int y)
  {
    int index=nearCity(x,y);
    if(index==-1) addCity(x,y);
             else deleteCity(index);
  }
  
  /** Computes minimal distance matrix.
    *
    * @see	#distance
    */
  public void computeDistanceMatrix()
  {
    for(int i=0; i<size; i++) 
      dist[i][i]=0;
    for(int i=0; i<size; i++)
      for(int j=0; j<i; j++)
        dist[i][j] =
        dist[j][i] = (int) Math.round( Math.sqrt(
          (x[i]-x[j])*(x[i]-x[j])+(y[i]-y[j])*(y[i]-y[j]) ));
    dmComputed=true;
  }
  
  /** Returns number of cities.
    *
    * @return	int - pocet mest
    */
  public int count()
  {
    return size;
  }
  
  /** Returns minimal distance between cities A and B.
    *
    * @param	a	city A
    * @param	b	city B
    * @return	minimal distance [A,B]
    * @see	computeDistanceMatrix
    */
  public int distance(int a, int b)
  {
    if(!dmComputed) return Integer.MAX_VALUE;
    return dist[a][b];
  }
  
  /** Resizes distance coordinates, ratio is to/from. */
  public void changeSize(int from, int to)
  {
    double factor=((double)(to))/((double)(from));
    for(int i=0; i<size; i++) 
      {
        x[i]=(int) ( factor*((double)(x[i])) );
        y[i]=(int) ( factor*((double)(y[i])) );
      }
  }
}